const mongoose = require('mongoose');
const Schema = mongoose.Schema;

const postTemplate = {
    author: String,
    body: String,
    date: Date,
}

const AD_TYPE = {
    VALUE: {
        SALE: "sale",
        RENTAL: "rental"
    },
    LANG: {
        SALE: "A la vente",
        RENTAL: "A la location"
    }
};

const AD_TRANSACTION_STATUS = {
    VALUE: {
        AVAILABLE: "available",
        RENTED: "rented",
        SOLD: "sold"
    },
    LANG: {
        AVAILABLE: "Disponible",
        RENTED: "Louée",
        SOLD: "Vendue"
    }
};

const adSchema = new Schema({
    title: {
        type: String,
        required: [true, 'Le titre est obligatoire !']
    },
    type: { 
        type: String,
        required: [true, 'Le type de bien est obligatoire !'],
        enum: Object.values(AD_TYPE.VALUE),
        get: l => AD_TYPE.LANG[l.toUpperCase()],
    },
    published: { 
        type: Boolean,
        required: [true, 'L\'état de publication est obligatoire !']
    },
    transactionStatus: { 
        type: String,
        required: [true, 'Le statut de la transaction est obligatoire !'],
        enum: Object.values(AD_TRANSACTION_STATUS.VALUE),
        get: l => AD_TRANSACTION_STATUS.LANG[l.toUpperCase()],
        validate: [
            _ => {
                if(this.type === AD_TYPE.VALUE.RENTAL) {
                    return _ !== AD_TRANSACTION_STATUS.VALUE.SOLD
                } else {
                    return _ !== AD_TRANSACTION_STATUS.VALUE.RENTED
                }
            },
            'Votre statut de transaction doit correspondre à la transaction..'
        ]
    },
    price: { 
        type: Number,
        required: [true, 'Le prix est obligatoire !'],
        validate: {
            validator: _ =>  {
                return String(_).match(/^\d+(\.\d{2})?$/);
            },
            msg: 'Votre prix doit ressembler à un prix !'
        }
    },
    description: String,
    availabilityDate: {
        type: Date,
        get: d => d && d.toLocaleDateString()
    },
    pictures: [{ body: Buffer, title: String }],
    questions: [{
        ...postTemplate,
        answers: [{
            ...postTemplate
        }]
    }]
});

module.exports = {
    AD_TYPE: AD_TYPE,
    AD_TRANSACTIONSTATUS: AD_TRANSACTION_STATUS,
    Ad: mongoose.model('Ad', adSchema)
}